/*
 ************************************************************************************************
 *                                                                                              *
 *  OSA cooperative RTOS for Microchip PIC-controllers: PIC10/12/16/18/24/dsPIC                 *
 *                                                                                              *
 *  URL:            http://wiki.pic24.ru/doku.php/en/osa/ref/intro                              *
 *                  http://picosa.narod.ru                                                      *
 *                                                                                              *
 *----------------------------------------------------------------------------------------------*
 *                                                                                              *
 *  File:           osa_mkc18.h                                                                 *
 *                                                                                              *
 *  Compilers:      HT-PICC STD                                                                 *
 *                  HT-PICC18 STD                                                               *
 *                  Microchip C18                                                               *
 *                  Microchip C30                                                               *
 *                                                                                              *
 *  Programmer:     Timofeev Victor                                                             *
 *                  osa@pic24.ru, testerplus@mail.ru                                            *
 *                                                                                              *
 *  Description:    mikroC for PIC18 specific definitions                                       *
 *                  The specific of mikroC cmpiler is that defined in macro inline asm {} will  *
 *                  be igored and not included into code.                                       *
 *                                                                                              *
 *  History:        21.09.2009                                                                  *
 *                                                                                              *
 ************************************************************************************************
 */
#ifndef __OSAMKC18_H__
#define __OSAMKC18_H__


#undef OS_SMSG_SIZE
#define OS_SMSG_SIZE    sizeof(OST_SMSG)



/*************************************************************************************************************************/
//------------------------------------------------------------------------------
//    FSR = FSR0
extern volatile unsigned int _fsr;
extern volatile char         _indf;
extern volatile char         _postinc;
extern volatile char         _postdec;
extern volatile char         _preinc;

extern volatile unsigned char _fsr1l;

extern volatile char         _pcl;
extern volatile char         _pclath;
extern volatile char         _pclatu;
extern volatile char         _status;
extern volatile char         _tosl;
extern volatile char         _tosh;
extern volatile char         _tosu;
extern volatile char         _bsr;
extern volatile char         _wreg;
extern volatile char         _intcon;
extern volatile char         _rcon;
extern volatile char         _stkptr;

#define _FSR_TYPE            unsigned int




//------------------------------------------------------------------------------
// Type of code pointer (used in OST_TCB)
#if defined(_ROMSIZE) && (_ROMSIZE <= 0x10000)
    #define OST_CODE_POINTER    unsigned int
    #define OS_CODE_POINTER_SIZE    2
#else
    #define OST_CODE_POINTER    unsigned long
    #define OS_CODE_POINTER_SIZE    4
#endif


//#define OST_CODE_POINTER    unsigned long

#define _OS_POP18()     _stkptr--


#define OS_CLRWDT()   asm(" clrwdt ");
#define OS_ClrWdt()   asm(" clrwdt ");
#define OS_SLEEP()    asm(" sleep  ");








//______________________________________________________________________________
/******************************************************************************
 * Switch control routines:
 *
 *  _OS_JumpToTask() - jump indirectly from kernel (OS_Sched) into task
 *  _OS_SaveRetAddr() - task return to this point
 *
 *  _OS_GetReturnPoint() - exit from task and save context
 *  _OS_CallGetReturnPoint() - save context without exiting task
 *  _OS_ReturnNoSave() - exit task without saving context
 *
 *******************************************************************************/


#if defined(_ROMSIZE) && (_ROMSIZE <= 0x10000)
    #define _OS_SetPC()                         \
        _OS_temp = _postinc;                    \
        _pclath =  _postinc;


    #define _OS_SavePC()                        \
        _preinc = _tosl;                        \
        _preinc = _tosh;                        \

#else

    #define _OS_SetPC()                         \
        _OS_temp = _postinc;                    \
        _pclath =  _postinc;                    \
        _pclatu =  _postinc;


    #define _OS_SavePC()                        \
        _preinc = _tosl;                        \
        _preinc = _tosh;                        \
        _preinc = _tosu;                        \

#endif

void __OS_JumpToTask (void);

    #define _OS_JumpToTask()                            \
        {                                               \
            __OS_JumpToTask();                          \
            _OS_SET_FSR_CUR_TASK();                     \
            _indf &= 0xD7;                              \
            _indf |= *((char*)&OS_state) & 0x28;        \
        }

    #define _OS_SaveRetAddr()

    #define _OS_if_Ready()  _OS_SET_FSR_CUR_TASK();                                 \
                            if (_OS_bTaskReady)


extern void OS_SchedRetPoint (void);
extern void OS_SchedRetPointNoSave (void);
extern void OS_ClearReady (void);
extern void OS_ClearReadySetCanContinue (void);

extern void  _OS_SET_FSR_CUR_TASK (void);


//------------------------------------------------------------------------------
#define _OS_GetReturnPoint()                    \
    {                                           \
        OS_SchedRetPoint();                     \
        asm {                                   \
            nop                                 \
        }                                       \
    }

//------------------------------------------------------------------------------
#define _OS_CLEAR_READY()                       \
    {                                           \
        OS_ClearReady();                        \
        asm {                                   \
            nop                                 \
        }                                       \
    }
//------------------------------------------------------------------------------
#define _OS_CLEAR_READY_SET_CANCONTINUE()       \
    {                                           \
        OS_ClearReadySetCanContinue();          \
        asm {                                   \
            nop                                 \
        }                                       \
    }
//------------------------------------------------------------------------------
#define _OS_ReturnNoSave()                      \
    {                                           \
        asm(" global _OS_SchedRetPointNoSave"); \
        OS_SchedRetPointNoSave();               \
        asm {                                   \
            nop                                 \
        }                                       \
    }                                           \
//------------------------------------------------------------------------------












/******************************************************************************************
 *
 *
 *
 ******************************************************************************************/





#endif





















